@@ -1,30 +1,25 @@
-import argparse
-import locale
-import pathlib
 import logging
 import os
 import sys
 import datetime
-import configparser
 import string
 import webbrowser
 import curses
 import curses.panel
 import traceback
 import tempfile
-import time
 import subprocess
 import shlex
 import shutil
 
-from pytodotxt import TodoTxt, Task, version
+from pytodotxt import TodoTxt, Task
 
 from cursedspace import Application, Panel, ScrollPanel, InputLine, ShellContext, Completion
 
 from pter import common
 from pter import utils
 from pter import configuration
-from pter.searcher import Searcher, get_relative_date
+from pter.searcher import Searcher
 from pter.key import Key
 from pter.tr import tr
 
@@ -117,15 +112,21 @@ class TaskLine:
 
     @property
     def is_overdue(self):
-        return self.due is not None and self.due < datetime.date.today() and not self.task.is_completed
+        return not self.task.is_completed and \
+               self.due is not None and \
+               self.due < datetime.date.today()
 
     @property
     def is_due_tomorrow(self):
-        return self.due is not None and self.due == datetime.date.today() + datetime.timedelta(days=1)
+        return not self.task.is_completed and \
+               self.due is not None and \
+               self.due == datetime.date.today() + datetime.timedelta(days=1)
 
     @property
     def is_due_today(self):
-        return self.due is not None and self.due == datetime.date.today()
+        return not self.task.is_completed and \
+               self.due is not None and \
+               self.due == datetime.date.today()
 
 
 class TaskLineDescription(TaskLineGroup):
@@ -182,7 +183,7 @@ class StatusBar(Panel):
         attr = self.app.color(common.SETTING_COL_NORMAL)
         try:
             self.win.addstr(0, 0, " "*self.dim[1], attr)
-        except:
+        except curses.error:
             pass
         if len(self.text) > 0:
             self.win.addstr(0, 0, self.text[:self.dim[1]], self.app.color(self.color))
@@ -192,7 +193,7 @@ class StatusBar(Panel):
 class HelpBar(Panel):
     def paint(self, clear=False):
         self.border = Panel.BORDER_NONE
-        super().paint(False)
+        super().paint(clear)
 
         mapping = self.app.key_mapping
         actions = ['show-help', 'quit', 'edit-task',
@@ -278,7 +279,7 @@ class TaskList(RemappedScrollPanel):
         self.update_max_width(common.TF_DUE, max([len(m) for m in self.app.due_marker]))
 
         for nr, pair in enumerate(self.tasks):
-            task, source = pair
+            task, _ = pair
             line = TaskLine(task, task.todotxt)
 
             # Selection indicator
@@ -291,7 +292,8 @@ class TaskList(RemappedScrollPanel):
             self.update_max_width(common.TF_NUMBER, text)
 
             # Done marker
-            line.add(common.TF_DONE, self.app.done_marker[1] if task.is_completed else self.app.done_marker[0])
+            done_marker = self.app.done_marker[1 if task.is_completed else 0]
+            line.add(common.TF_DONE, done_marker)
 
             # Age
             if task.creation_date is not None:
@@ -323,11 +325,17 @@ class TaskList(RemappedScrollPanel):
 
                 if not task.is_completed:
                     if line.due < today:
-                        line.add(common.TF_DUE, self.app.due_marker[0], common.SETTING_COL_OVERDUE)
+                        line.add(common.TF_DUE,
+                                 self.app.due_marker[0],
+                                 common.SETTING_COL_OVERDUE)
                     elif line.due == today:
-                        line.add(common.TF_DUE, self.app.due_marker[1], common.SETTING_COL_DUE_TODAY)
+                        line.add(common.TF_DUE,
+                                 self.app.due_marker[1],
+                                 common.SETTING_COL_DUE_TODAY)
                     elif line.due == today + datetime.timedelta(days=1):
-                        line.add(common.TF_DUE, self.app.due_marker[2], common.SETTING_COL_DUE_TOMORROW)
+                        line.add(common.TF_DUE,
+                                 self.app.due_marker[2],
+                                 common.SETTING_COL_DUE_TOMORROW)
 
             # Priority marker
             if task.priority is not None:
@@ -372,7 +380,8 @@ class TaskList(RemappedScrollPanel):
                                 attr = common.SETTING_COL_PRI_C
                     description.append(word, attr, space_around=True)
             line.elements[common.TF_DESCRIPTION] = description
-            self.update_max_width(common.TF_DESCRIPTION, ' '.join([e.content for e in description.elements]))
+            self.update_max_width(common.TF_DESCRIPTION,
+                                  ' '.join([e.content for e in description.elements]))
 
             self.items.append(line)
         self.cursor = min(self.cursor, len(self.items)-1)
@@ -396,7 +405,6 @@ class TaskList(RemappedScrollPanel):
         super().paint(clear)
 
     def do_paint_item(self, y, x, maxwidth, is_selected, taskline):
-        line = ''
         is_tracked = len(taskline.task.attr_tracking) > 0
 
         baseattrs = common.SETTING_COL_NORMAL
@@ -466,8 +474,14 @@ class TaskList(RemappedScrollPanel):
                 line = extra[0]
 
             if align is not None and align.endswith('>'):
-                spacing = align[0] * min(maxwidth - len(line), self.max_widths[group.name] - len(' '.join([e.content for e in group.elements])))
-                self.scr.addstr(y, x + len(line), spacing, self.app.color(baseattrs, is_selected))
+                group_width = len(' '.join([e.content for e in group.elements]))
+                spacing_width = min(maxwidth - len(line),
+                                    self.max_widths[group.name] - group_width)
+                spacing = align[0] * spacing_width
+                try:
+                    self.win.addstr(y, x + len(line), spacing, self.app.color(baseattrs, is_selected))
+                except curses.error:
+                    pass
                 line += spacing
 
             cut_off = False
@@ -478,16 +492,21 @@ class TaskList(RemappedScrollPanel):
                     if len(line) + 1 >= maxwidth:
                         cut_off = True
                     else:
-                        self.win.addstr(y, x + len(line), " ", self.app.color(baseattrs, is_selected))
+                        self.win.addstr(y, x + len(line), " ",
+                                        self.app.color(baseattrs, is_selected))
                         line += " "
 
                 cut_off = cut_off or len(line) >= maxwidth
 
                 if not cut_off:
                     if isinstance(element, TaskLineGroup):
-                        word = print_group(y, x + len(line), maxwidth-len(line), element, None, None)
+                        word = print_group(y, x + len(line),
+                                           maxwidth-len(line),
+                                           element, None, None)
                     else:
-                        word = print_element(y, x + len(line), maxwidth-len(line), element, None, None)
+                        word = print_element(y, x + len(line),
+                                             maxwidth-len(line),
+                                             element, None, None)
                     line += word
 
                 if cut_off or len(line) >= maxwidth:
@@ -495,8 +514,11 @@ class TaskList(RemappedScrollPanel):
 
             if not cut_off and align is not None and align.endswith('<'):
                 if len(line) < self.max_widths[group.name]:
-                    spacing = align[0] * min(maxwidth-len(line), self.max_widths[group.name]-len(line))
-                    self.win.addstr(y, x + len(line), spacing, self.app.color(baseattrs, is_selected))
+                    spacing = align[0] * min(maxwidth-len(line),
+                                             self.max_widths[group.name]-len(line))
+                    self.win.addstr(y, x + len(line),
+                                    spacing,
+                                    self.app.color(baseattrs, is_selected))
                     line += spacing
 
             if not cut_off and extra is not None and extra[1] is not None and len(line) + 1 < maxwidth:
@@ -533,23 +555,25 @@ class TaskList(RemappedScrollPanel):
     def jump_to(self, item):
         if item in self.items:
             self.cursor = self.items.index(item)
-        
+
         elif isinstance(item, int):
             if item < 0 or item >= len(self.items):
                 return False
             self.cursor = item
-        
+
         elif isinstance(item, (TaskLine, Task)):
             if isinstance(item, TaskLine):
                 task = str(item.source.filename) + "\n" + str(item.task)
             elif isinstance(item, Task):
                 task = str(item.todotxt.filename) + "\n" + str(item)
+            else:
+                return False
 
             matching = [idx for idx, t in enumerate(self.items)
                         if str(t.source.filename) + "\n" + str(t.task) == task]
 
             if len(matching) == 0:
-                return
+                return False
             self.cursor = matching[0]
 
         else:
@@ -707,7 +731,7 @@ class SearchBar(RemappedInputLine):
         visible_text = visible_text[:self.dim[1]]
         try:
             self.win.addstr(0, 0, " "*self.dim[1])
-        except:
+        except curses.error:
             pass
         self.win.addstr(0, 0, visible_text, attr)
         self.win.noutrefresh()
@@ -1188,6 +1212,8 @@ class CursesApplication(Application):
         self.clear_contexts = [context
                                for context in conf.list(common.SETTING_GROUP_GENERAL, common.SETTING_CLEAR_CONTEXT)
                                if len(context) > 0]
+        self.reuse_recurring = conf.bool(common.SETTING_GROUP_GENERAL,
+                                         common.SETTING_REUSE_RECURRING, 'n')
         self.scroll_margin = conf.number(common.SETTING_GROUP_GENERAL,
                                          common.SETTING_SCROLL_MARGIN)
         self.safe_save = conf.bool(common.SETTING_GROUP_GENERAL,
@@ -1216,9 +1242,8 @@ class CursesApplication(Application):
 
         # ensure the trash file can be written
         if self.delete_is == common.DELETE_OPTION_TRASH:
-            failed = None
             try:
-                with open(self.trash_file, 'a'):
+                with open(self.trash_file, 'a', encoding='utf-8'):
                     logging.debug(f"Using trash file {self.trash_file}")
             except OSError as exc:
                 logging.error(f"Could not open trash file '{self.trash_file}': {exc}")
@@ -1431,7 +1456,7 @@ class CursesApplication(Application):
         attr = self.color(common.SETTING_COL_NORMAL)
         try:
             self.screen.addstr(1, 0, "─"*self.size()[1], attr)
-        except:
+        except curses.error:
             pass
         self.screen.noutrefresh()
 
@@ -1533,7 +1558,7 @@ class CursesApplication(Application):
                 self.color(colorname, 0)
                 self.color(colorname, 1)
 
-            for number, key in enumerate(self.conf[common.SETTING_GROUP_HIGHLIGHT]):
+            for key in self.conf[common.SETTING_GROUP_HIGHLIGHT]:
                 hlcol = Color(*self.conf.color_pair(common.SETTING_GROUP_HIGHLIGHT, key))
 
                 variant = 0
@@ -1646,7 +1671,7 @@ class CursesApplication(Application):
 
         to_exit = [k for k, fnc in self.key_mapping.items() if fnc == 'quit']
         if len(to_exit) == 0:
-            logging.fatal(f"No key defined to exit pter.")
+            logging.fatal("No key defined to exit pter.")
             raise RuntimeError("No key to exit")
 
     def info(self, text):
@@ -1690,6 +1715,7 @@ class CursesApplication(Application):
             self.update_tasks()
             self.tasks.jump_to(task)
             return True, task
+        self.update_tasks()
         self.error(tr("Not changed: task was modified in the background"))
         return False, None
 
@@ -1901,16 +1927,16 @@ class CursesApplication(Application):
     def do_toggle_done(self):
         if self.tasks.selected_item is None:
             return
-        success, task = self.modify_task(self.tasks.selected_item,
-                                         lambda t: utils.toggle_done(t, self.clear_contexts))
+        success, _ = self.modify_task(self.tasks.selected_item,
+                                      lambda t: utils.toggle_done(t, self.clear_contexts, self.reuse_recurring, self.safe_save))
         if success:
             self.tasks.paint(True)
 
     def do_toggle_hidden(self):
         if self.tasks.selected_item is None:
             return
-        success, task = self.modify_task(self.tasks.selected_item,
-                                         lambda t: utils.toggle_hidden(t))
+        success, _ = self.modify_task(self.tasks.selected_item,
+                                      lambda t: utils.toggle_hidden(t))
         if success:
             self.tasks.paint(True)
 
