@@ -42,6 +42,15 @@ def parse_args(is_qtui):
                         type=str,
                         default=None,
                         help=tr("Add this as a new task to the selected file."))
+    parser.add_argument('-l', '--log-level',
+                        type=str,
+                        default='error',
+                        help=tr("Log level. Defaults to %(default)s."))
+    parser.add_argument('--log-file',
+                        type=str,
+                        default=common.LOGFILE,
+                        help=tr("Where to write the log file to. Defaults to %(default)s."))
+
     if is_qtui:
         parser.add_argument('-a', '--add-task',
                             default=False,
@@ -111,7 +120,7 @@ def run():
         success = run_cursesui(args)
 
     else:
-        print(tr("Neither PyQt5 nor curses are installed."), file=sys.stderr)
+        print(tr("Neither PyQt5 nor curses are installed. To start the Qt version, please run 'qpter'."), file=sys.stderr)
         success = -2
 
     return success
