@@ -452,7 +452,7 @@ class Searcher:
             return any([parent in parents for parent in self.after])
 
 
-def get_relative_date(text, fmt=None):
+def get_relative_date(text, fmt=None, base=None):
     weekdays = ['mon', 'tue', 'wed', 'thu', 'fri',
                 'sat', 'sun']
     days = 0
@@ -461,24 +461,24 @@ def get_relative_date(text, fmt=None):
     years = 0
     value = ''
     today = datetime.datetime.now()
+    if base is not None:
+        assert isinstance(base, datetime.date)
+        today = base
 
+    then = None
     if text.lower() == 'today':
-        if fmt is not None:
-            return today.strftime(fmt)
-        return today
+        then = today
     if text.lower() == 'tomorrow':
         then = today + datetime.timedelta(days=1)
-        if fmt is not None:
-            return then.strftime(fmt)
-        return then
     if text.lower() in ['next-week', 'week']:
-        return today + datetime.timedelta(weeks=1)
+        then = today + datetime.timedelta(weeks=1)
     if text.lower() in weekdays:
         index = weekdays.index(text.lower())
         days = (index - today.weekday()) % 7
         if days == 0:
             days = 7
         then = today + datetime.timedelta(days=days)
+    if then is not None:
         if fmt is not None:
             return then.strftime(fmt)
         return then
@@ -489,7 +489,12 @@ def get_relative_date(text, fmt=None):
         elif char in '+-' and len(value) == 0:
             if char == '-':
                 value = char
-        elif value.isnumeric() or (value.startswith('-') and value[1:].isnumeric()):
+        elif len(value) == 0 or \
+             value.isnumeric() or \
+             (value.startswith('-') and value[1:].isnumeric()):
+            if len(value) == 0:
+                value = '1'
+
             if char == 'd':
                 days += int(value)
             elif char == 'w':
@@ -525,4 +530,3 @@ def get_relative_date(text, fmt=None):
     if fmt is not None:
         then = then.strftime(fmt)
     return then
-
