@@ -116,11 +116,12 @@ def dehumanize_dates(text, tags=None):
 def ensure_up_to_date(task):
     ok = True
     if task.todotxt.refresh():
-        ok = False
         task.todotxt.parse()
-        tasks = [other for other in source.tasks if other.raw.strip() == task.raw.strip()]
-        if len(tasks) > 0:
-            ok = True
+        tasks = [other
+                 for other in task.todotxt.tasks
+                 if str(other).strip() == str(task).strip()]
+        ok = len(tasks) > 0
+        if ok:
             task = tasks[0]
 
     if ok:
@@ -136,7 +137,59 @@ def toggle_tracking(task):
     return True
 
 
-def toggle_done(task, clear_contexts):
+def toggle_done(task, clear_contexts, reuse_recurring, safe_save):
+    """Toggle the "done" status of the given task
+
+    This may, in case of 'rec:' tags, create a follow-up task in the future
+    and thus update the corresponding todo.txt file.
+    """
+    is_recurring = 'rec' in task.attributes
+
+    if not task.is_completed and is_recurring:
+        interval = task.attributes['rec']
+        has_due = task.attributes.get('due')
+        has_t = task.attributes.get('t')
+
+        interval = interval[0] if len(interval) > 0 else None
+
+        is_strict = interval.startswith('+')
+        base_date = datetime.date.today()
+
+        if is_strict:
+            if has_due is not None:
+                base_date = datetime.datetime.strptime(has_due[0], Task.DATE_FMT).date()
+            elif has_t is not None:
+                base_date = datetime.datetime.strptime(has_t[0], Task.DATE_FMT).date()
+
+        next_due = get_relative_date(interval, None, base_date)
+
+        if reuse_recurring:
+            followup = task
+        else:
+            followup = Task(str(task), len(task.todotxt.tasks)+1, todotxt=task.todotxt)
+            task.todotxt.tasks.append(followup)
+
+        # update attributes
+        followup.creation_date = datetime.date.today()
+
+        if has_due is not None:
+            followup.replace_attribute('due', has_due[0], next_due.strftime(Task.DATE_FMT))
+        elif has_t is not None:
+            followup.replace_attribute('t', has_t[0], next_due.strftime(Task.DATE_FMT))
+
+        if None not in [has_t, has_due]:
+            time_between_t_and_due = datetime.datetime.strptime(has_due[0], Task.DATE_FMT).date() \
+                                      - datetime.datetime.strptime(has_t[0], Task.DATE_FMT).date()
+            next_t = next_due - time_between_t_and_due
+            followup.replace_attribute('t', has_t[0], next_t.strftime(Task.DATE_FMT))
+
+        if reuse_recurring:
+            task.parse(str(task))
+            return
+
+        # create the new task in the same todotxt file
+        task.todotxt.save(safe_save)
+
     task.is_completed = not task.is_completed
     if task.is_completed:
         task.completion_date = datetime.datetime.now().date()
@@ -219,8 +272,6 @@ def update_spent(task):
     if len(raw_spent) > 0:
         spent = parse_duration(raw_spent[0])
         if spent is None:
-            self.status_bar.addstr(0, 0, "Failed to parse 'spent' time", self.color(Window.ERROR))
-            self.status_bar.noutrefresh()
             return False
     else:
         spent = datetime.timedelta(0)
@@ -254,6 +305,8 @@ def human_friendly_date(text):
             then = datetime.datetime.strptime(text, Task.DATE_FMT).date()
         except ValueError:
             return text
+    else:
+        return text
     today = datetime.date.today()
     diff = abs((then - today).days)
     then_wd = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday',
@@ -368,7 +421,7 @@ def parse_searches():
 
     searches = {}
     with open(common.SEARCHES_FILE, 'rt', encoding="utf-8") as fd:
-        for linenr, line in enumerate(fd.readlines()):
+        for line in fd.readlines():
             if '=' not in line:
                 continue
             name, searchdef = line.split("=", 1)
@@ -387,7 +440,7 @@ def parse_templates():
 
     templates = {}
     with open(common.TEMPLATES_FILE, 'rt', encoding="utf-8") as fd:
-        for linenr, line in enumerate(fd.readlines()):
+        for line in fd.readlines():
             if '=' not in line:
                 continue
             name, templatedef = line.split("=", 1)
@@ -478,7 +531,8 @@ def new_task_id(sources, prefix=""):
 def query_latest_version():
     installed_re = re.compile(r'^[\s]+IN_STALLED:[\s]+([^\s]+)(.*)$')
     latest_re = re.compile(r'^[\s]+LATEST:[\s]+([^\s]+)$')
-    output = subprocess.run([sys.executable, "-m", "pip", "search", common.PROGRAMNAME], capture_output=True)
+    output = subprocess.run([sys.executable, "-m", "pip", "search",
+                             common.PROGRAMNAME], capture_output=True)
     for line in str(output.stdout, 'utf-8').split("\n"):
         match = installed_re.match(line)
         if match is not None:
@@ -513,8 +567,7 @@ def auto_task_id(sources, text):
             _, base = word.split(':', 1)
             if '#' in base:
                 base, keyword = base.split('#', 1)
-                if keyword == '' or keyword == 'auto':
+                if keyword in ['', 'auto']:
                     word = 'id:{}'.format(new_task_id(sources, base))
         words.append(word)
     return ' '.join(words)
-
